import java.applet.Applet;
import java.awt.event.*;
import java.awt.*;

/**This deals with the basic object, a power combo.
The triple (c,b,t) determines the function 

  f(s)= c s^t b^(s/2)

We get the big function F by summing these terms over all triples.
The values [L,R] keep track of the domain of s*/

public class PowerCombo {
    int COUNT; //number of terms
    BigRational[] c=new BigRational[100]; //the coefficients
    BigRational[] b=new BigRational[100]; //the base of the exponent
    int[] t=new int[100];//the shift in the exponent
    BigRational L,R; //the endpoints of the interval;


    /**initializes the object*/
    public PowerCombo() {}

    /**copies the object*/
    public PowerCombo(PowerCombo A) {
	COUNT=A.COUNT;
	for(int i=0;i<COUNT;++i) {
	    c[i]=A.c[i];
	    t[i]=A.t[i];
	    b[i]=new BigRational(A.b[i]);
	}
	L=new BigRational(A.L);
	R=new BigRational(A.R);
    }
    

    /**subdivision operations*/
    
    public PowerCombo sub0() {
	PowerCombo A=new PowerCombo(this);
	A.L=new BigRational(L);
	A.R=L.add(R);
	A.R=A.R.multiply(new BigRational("1","2"));
	return A;
    }
    
    public PowerCombo sub1() {
	PowerCombo A=new PowerCombo(this);
	A.R=new BigRational(R);
	A.L=L.add(R);
	A.L=A.L.multiply(new BigRational("1","2"));
	return A;
    }
    /**end subdivision operations*/

    

    /**positivity test*/
    public boolean isPositive() {
	for(int i=0;i<2;++i) {
	    BigRational A=evaluateLower();
	   if(A.isPositive()==false) return false;
	}
	return true;
    }



    /**This is the main routine.  It tests if our power combos
       are positive in the interval [L,R], cutting off if R<L0 or L>R0;
       When [L,R]=[0,6] we take X=1/1024.  When [L,R]=[6,16] we 
       take X=0*/
    
    public static void testPositive(PowerCombo A,BigRational L,BigRational R,BigRational L0,BigRational R0) {
	A.L=new BigRational(L);
	A.R=new BigRational(R);
	PowerCombo[] LIST=new PowerCombo[50];
	LIST[0]=new PowerCombo(A);
	int COUNT=1;
	boolean HALT=false;

	System.out.print("start ... ");
	while(HALT==false) {
	    PowerCombo Q=LIST[COUNT-1];
	    boolean test=Q.isPositiveWithCutoff(L0,R0);
	    if(test==true) {
		--COUNT;
	    }
	    
	    else {
		LIST[COUNT-1]=Q.sub0();
		LIST[COUNT]=Q.sub1();
		++COUNT;
	    }
	    if(COUNT==0) HALT=true;
	}
	System.out.println("done");
    }


    /**This tests if the power combo is positive on the interval,
       [X,R] whatever the uppwe bound R is.  We always take R=6*/
    
    public boolean isPositiveWithCutoff(BigRational L0,BigRational R0) {
	BigRational test=new BigRational(0,1);
	test=R.subtract(L0);
	if(test.isNegative()==true) return true;
	test=R0.subtract(L);
	if(test.isNegative()==true) return true;

	//now we know the interval is in range
	
	for(int i=0;i<2;++i) {
	    BigRational A=evaluateLower();
	   if(A.isPositive()==false) return false;
	}
	return true;
    }

    /**This computes a lower bound.*/
    
    public BigRational evaluateLower() {
	BigRational L2=L.multiply(new BigRational("1","2"));
	BigRational R2=R.multiply(new BigRational("1","2"));

	BigRational SUM=new BigRational("0","1");
	
	for(int i=0;i<COUNT;++i) {
	    BigRational a=valueLower(-1,c[i],b[i],t[i],L,R);
	    SUM=SUM.add(a);
	}
	return SUM;
    }

    /**returns the minimum value of the 4 evaluations*/

    public static BigRational valueLower(int sign,BigRational c,BigRational b,int t,BigRational L,BigRational R) {
	BigRational a1=value(+1,c,b,t,L);
	BigRational a2=value(-1,c,b,t,L);
	BigRational a3=value(+1,c,b,t,R);
	BigRational a4=value(-1,c,b,t,R);
	a1=BigRational.min(a1,a2);
	a1=BigRational.min(a1,a3);
	a1=BigRational.min(a1,a4);
	return a1;
    }

    /**This evaluates c s^t b^{s/2} for s=E.  The sign decides whether we
       take an under-approximation or an over-approximation*/
    
    public static BigRational value(int sign,BigRational c,BigRational b,int t,BigRational E) {
	    BigRational E2=E.multiply(new BigRational("1","2"));
	    BigRational a=RigorousPower.pow(sign,b,E2);
	    a=a.multiply(c);
	    for(int i=0;i<t;++i) a=a.multiply(E);
	    return a;
    }

    /**prints out the interval: for debugging purposes*/
    public void printInterval() {
	L.print();
	System.out.print("   ");
	R.println();
    }
}
	
	


