import java.math.*;

public class TaylorSeries {

    /**The goal of this class is to get under and over approximations
       to expressions of the form

       Y0 2^{-x/2}+
       Y1 3^{-x/2}+
       Y2 4^{-x/2}+
       Y3 x 2^{-x/2}+
       Y4 x 3^{-x/2}+
       Y5 x 4^{-x/2}.

       on the interval [0,1]. */

    
    /**Here is the main expression. The variable choice refers
       to whether we are over or under approximating:
       0 under, 1 over*/

    public static Poly1 main(long[] Y) {
	Poly1Interval P=combo(Y);
	Poly1 Q=P.min();
	return Q;
    }
    

    /**These are the under and over approximations of log m:
       j=0: under approx for log(m)
       j=1: over approx for log(m)
       This is only meant to be used when m=2,3,4.*/

    public static BigRational LOG(int m,int j) {
	long[][] p={{25469,5225,25469},{7050,708784,345197}};
	long[][] q={{36744,4756,18372},{10171,645163,249007}};
	return new BigRational(p[j][m-2],q[j][m-2]);
    }

    public static BigRationalInterval LOG(int m) {
	BigRational L=LOG(m,0);
	BigRational R=LOG(m,1);
	BigRationalInterval Y=new BigRationalInterval(L,R);
	return Y;
    }
    /**end approximation for log*/


    

    /**(i)th Taylor series coefficient for Power[m,-s/2] 
       centered at 0.*/

    public static BigRationalInterval seriesTerm(int m,int i) {
	BigRational TWO=new BigRational(2,1);
	BigRational M=new BigRational(m,1);
        BigRationalInterval L=LOG(m);
	BigRational T=new BigRational(1,1);
        if(i%2==1) T=new BigRational(-1,1);
	T=T.divide(BigRational.factorial(i));
	T=T.divide(BigRational.power(TWO,i));
	L=BigRationalInterval.powerBasic(L,i);
	L=L.scale(T);
	return L;
    }


    /**This adds these up to get a series*/
    public static Poly1Interval series(long c,int m) {
	Poly1Interval P=new Poly1Interval();	
        P.degree=5;
	for(int i=0;i<5;++i) P.C[i]=seriesTerm(m,i);
	BigRational C=new BigRational(c,1);
	for(int i=0;i<5;++i) P.C[i]=P.C[i].scale(C);

	//the error term
        BigRational S=BigRational.factorial(5);
	BigRational E1=new BigRational(-1,1);
	BigRational E2=new BigRational(+1,1);	
        E1=E1.divide(S);
        E2=E2.divide(S);
	P.C[5]=new BigRationalInterval(E1,E2);
	
	return P;
    }

    
    /**This builds the series underapproximation for the power combo.
       This is the lowest degree approximation that will do the job*/
    
    public static Poly1Interval combo(long[] Y) {
	Poly1Interval P=new Poly1Interval();
	Poly1Interval Q=new Poly1Interval();
	
	P.degree=5;
       	Q.degree=5;
	
	Q=series(Y[0],2);
       	P=P.add(Q);
	
	Q=series(Y[1],3);
       	P=P.add(Q);
	
	Q=series(Y[2],4);
       	P=P.add(Q);
	
	Q=series(Y[3],2);
	Q.shiftUp();
       	P=P.add(Q);

	Q=series(Y[4],3);
	Q.shiftUp();
       	P=P.add(Q);
	
	Q=series(Y[5],4);
	Q.shiftUp();
       	P=P.add(Q);

	P.degree=6;
	return P;
    }

}

